@@ -1,5 +1,5 @@
 # orm/collections.py
-# Copyright (C) 2005-2016 the SQLAlchemy authors and contributors
+# Copyright (C) 2005-2019 the SQLAlchemy authors and contributors
 # <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
@@ -103,19 +103,23 @@ through the adapter, allowing for some very sophisticated behavior.
 
 """
 
-import inspect
 import operator
 import weakref
 
-from ..sql import expression
-from .. import util, exc as sa_exc
+from sqlalchemy.util.compat import inspect_getfullargspec
 from . import base
+from .. import exc as sa_exc
+from .. import util
+from ..sql import expression
 
-from sqlalchemy.util.compat import inspect_getargspec
 
-__all__ = ['collection', 'collection_adapter',
-           'mapped_collection', 'column_mapped_collection',
-           'attribute_mapped_collection']
+__all__ = [
+    "collection",
+    "collection_adapter",
+    "mapped_collection",
+    "column_mapped_collection",
+    "attribute_mapped_collection",
+]
 
 __instrumentation_mutex = util.threading.Lock()
 
@@ -172,10 +176,12 @@ class _SerializableColumnGetter(object):
     def __call__(self, value):
         state = base.instance_state(value)
         m = base._state_mapper(state)
-        key = [m._get_state_attr_by_column(
-            state, state.dict,
-            m.mapped_table.columns[k])
-            for k in self.colkeys]
+        key = [
+            m._get_state_attr_by_column(
+                state, state.dict, m.mapped_table.columns[k]
+            )
+            for k in self.colkeys
+        ]
         if self.composite:
             return tuple(key)
         else:
@@ -208,16 +214,15 @@ class _SerializableColumnGetterV2(_PlainColumnGetter):
                 return None
             else:
                 return c.table.key
+
         colkeys = [(c.key, _table_key(c)) for c in cols]
         return _SerializableColumnGetterV2, (colkeys,)
 
     def _cols(self, mapper):
         cols = []
-        metadata = getattr(mapper.local_table, 'metadata', None)
+        metadata = getattr(mapper.local_table, "metadata", None)
         for (ckey, tkey) in self.colkeys:
-            if tkey is None or \
-                    metadata is None or \
-                    tkey not in metadata:
+            if tkey is None or metadata is None or tkey not in metadata:
                 cols.append(mapper.local_table.c[ckey])
             else:
                 cols.append(metadata.tables[tkey].c[ckey])
@@ -237,9 +242,10 @@ def column_mapped_collection(mapping_spec):
     after a session flush.
 
     """
-    cols = [expression._only_column_elements(q, "mapping_spec")
-            for q in util.to_list(mapping_spec)
-            ]
+    cols = [
+        expression._only_column_elements(q, "mapping_spec")
+        for q in util.to_list(mapping_spec)
+    ]
     keyfunc = _PlainColumnGetter(cols)
     return lambda: MappedCollection(keyfunc)
 
@@ -253,7 +259,7 @@ class _SerializableAttrGetter(object):
         return self.getter(target)
 
     def __reduce__(self):
-        return _SerializableAttrGetter, (self.name, )
+        return _SerializableAttrGetter, (self.name,)
 
 
 def attribute_mapped_collection(attr_name):
@@ -311,6 +317,7 @@ class collection(object):
         def popitem(self): ...
 
     """
+
     # Bundled as a class solely for ease of use: packaging, doc strings,
     # importability.
 
@@ -355,7 +362,7 @@ class collection(object):
         promulgation to collection events.
 
         """
-        fn._sa_instrument_role = 'appender'
+        fn._sa_instrument_role = "appender"
         return fn
 
     @staticmethod
@@ -382,7 +389,7 @@ class collection(object):
         promulgation to collection events.
 
         """
-        fn._sa_instrument_role = 'remover'
+        fn._sa_instrument_role = "remover"
         return fn
 
     @staticmethod
@@ -396,7 +403,7 @@ class collection(object):
             def __iter__(self): ...
 
         """
-        fn._sa_instrument_role = 'iterator'
+        fn._sa_instrument_role = "iterator"
         return fn
 
     @staticmethod
@@ -421,6 +428,13 @@ class collection(object):
         return fn
 
     @staticmethod
+    @util.deprecated(
+        "1.0",
+        "The :meth:`.collection.linker` handler is deprecated and will "
+        "be removed in a future release.  Please refer to the "
+        ":meth:`.AttributeEvents.init_collection` "
+        "and :meth:`.AttributeEvents.dispose_collection` event handlers. ",
+    )
     def linker(fn):
         """Tag the method as a "linked to attribute" event handler.
 
@@ -430,18 +444,27 @@ class collection(object):
         the instance.  A single argument is passed: the collection adapter
         that has been linked, or None if unlinking.
 
-        .. deprecated:: 1.0.0 - the :meth:`.collection.linker` handler
-           is superseded by the :meth:`.AttributeEvents.init_collection`
-           and :meth:`.AttributeEvents.dispose_collection` handlers.
 
         """
-        fn._sa_instrument_role = 'linker'
+        fn._sa_instrument_role = "linker"
         return fn
 
     link = linker
-    """deprecated; synonym for :meth:`.collection.linker`."""
+    """Synonym for :meth:`.collection.linker`.
+
+    .. deprecated:: 1.0 - :meth:`.collection.link` is deprecated and will be
+       removed in a future release.
+
+    """
 
     @staticmethod
+    @util.deprecated(
+        "1.3",
+        "The :meth:`.collection.converter` handler is deprecated and will "
+        "be removed in a future release.  Please refer to the "
+        ":class:`.AttributeEvents.bulk_replace` listener interface in "
+        "conjunction with the :func:`.event.listen` function.",
+    )
     def converter(fn):
         """Tag the method as the collection converter.
 
@@ -471,7 +494,7 @@ class collection(object):
         validation on the values about to be assigned.
 
         """
-        fn._sa_instrument_role = 'converter'
+        fn._sa_instrument_role = "converter"
         return fn
 
     @staticmethod
@@ -490,9 +513,11 @@ class collection(object):
             def do_stuff(self, thing, entity=None): ...
 
         """
+
         def decorator(fn):
-            fn._sa_instrument_before = ('fire_append_event', arg)
+            fn._sa_instrument_before = ("fire_append_event", arg)
             return fn
+
         return decorator
 
     @staticmethod
@@ -510,10 +535,12 @@ class collection(object):
             def __setitem__(self, index, item): ...
 
         """
+
         def decorator(fn):
-            fn._sa_instrument_before = ('fire_append_event', arg)
-            fn._sa_instrument_after = 'fire_remove_event'
+            fn._sa_instrument_before = ("fire_append_event", arg)
+            fn._sa_instrument_after = "fire_remove_event"
             return fn
+
         return decorator
 
     @staticmethod
@@ -532,9 +559,11 @@ class collection(object):
         collection.removes_return.
 
         """
+
         def decorator(fn):
-            fn._sa_instrument_before = ('fire_remove_event', arg)
+            fn._sa_instrument_before = ("fire_remove_event", arg)
             return fn
+
         return decorator
 
     @staticmethod
@@ -552,13 +581,15 @@ class collection(object):
         collection.remove.
 
         """
+
         def decorator(fn):
-            fn._sa_instrument_after = 'fire_remove_event'
+            fn._sa_instrument_after = "fire_remove_event"
             return fn
+
         return decorator
 
 
-collection_adapter = operator.attrgetter('_sa_adapter')
+collection_adapter = operator.attrgetter("_sa_adapter")
 """Fetch the :class:`.CollectionAdapter` for a collection."""
 
 
@@ -576,7 +607,13 @@ class CollectionAdapter(object):
     """
 
     __slots__ = (
-        'attr', '_key', '_data', 'owner_state', '_converter', 'invalidated')
+        "attr",
+        "_key",
+        "_data",
+        "owner_state",
+        "_converter",
+        "invalidated",
+    )
 
     def __init__(self, attr, owner_state, data):
         self.attr = attr
@@ -675,9 +712,8 @@ class CollectionAdapter(object):
             if self.invalidated:
                 self._warn_invalidated()
             return self.attr.fire_append_event(
-                self.owner_state,
-                self.owner_state.dict,
-                item, initiator)
+                self.owner_state, self.owner_state.dict, item, initiator
+            )
         else:
             return item
 
@@ -693,9 +729,8 @@ class CollectionAdapter(object):
             if self.invalidated:
                 self._warn_invalidated()
             self.attr.fire_remove_event(
-                self.owner_state,
-                self.owner_state.dict,
-                item, initiator)
+                self.owner_state, self.owner_state.dict, item, initiator
+            )
 
     def fire_pre_remove_event(self, initiator=None):
         """Notify that an entity is about to be removed from the collection.
@@ -707,22 +742,29 @@ class CollectionAdapter(object):
         if self.invalidated:
             self._warn_invalidated()
         self.attr.fire_pre_remove_event(
-            self.owner_state,
-            self.owner_state.dict,
-            initiator=initiator)
+            self.owner_state, self.owner_state.dict, initiator=initiator
+        )
 
     def __getstate__(self):
-        return {'key': self._key,
-                'owner_state': self.owner_state,
-                'data': self.data}
+        return {
+            "key": self._key,
+            "owner_state": self.owner_state,
+            "owner_cls": self.owner_state.class_,
+            "data": self.data,
+            "invalidated": self.invalidated,
+        }
 
     def __setstate__(self, d):
-        self._key = d['key']
-        self.owner_state = d['owner_state']
-        self._data = weakref.ref(d['data'])
+        self._key = d["key"]
+        self.owner_state = d["owner_state"]
+        self._data = weakref.ref(d["data"])
+        self._converter = d["data"]._sa_converter
+        d["data"]._sa_adapter = self
+        self.invalidated = d["invalidated"]
+        self.attr = getattr(d["owner_cls"], self._key).impl
 
 
-def bulk_replace(values, existing_adapter, new_adapter):
+def bulk_replace(values, existing_adapter, new_adapter, initiator=None):
     """Load a new collection, firing events based on prior like membership.
 
     Appends instances in ``values`` onto the ``new_adapter``. Events will be
@@ -753,14 +795,13 @@ def bulk_replace(values, existing_adapter, new_adapter):
 
     for member in values or ():
         if member in additions:
-            appender(member)
+            appender(member, _sa_initiator=initiator)
         elif member in constants:
             appender(member, _sa_initiator=False)
 
     if existing_adapter:
-        remover = existing_adapter.bulk_remover()
         for member in removals:
-            remover(member)
+            existing_adapter.fire_remove_event(member, initiator=initiator)
 
 
 def prepare_instrumentation(factory):
@@ -790,7 +831,7 @@ def prepare_instrumentation(factory):
     # Instrument the class if needed.
     if __instrumentation_mutex.acquire():
         try:
-            if getattr(cls, '_sa_instrumented', None) != id(cls):
+            if getattr(cls, "_sa_instrumented", None) != id(cls):
                 _instrument_class(cls)
         finally:
             __instrumentation_mutex.release()
@@ -823,10 +864,11 @@ def _instrument_class(cls):
     # In the normal call flow, a request for any of the 3 basic collection
     # types is transformed into one of our trivial subclasses
     # (e.g. InstrumentedList).  Catch anything else that sneaks in here...
-    if cls.__module__ == '__builtin__':
+    if cls.__module__ == "__builtin__":
         raise sa_exc.ArgumentError(
             "Can not instrument a built-in type. Use a "
-            "subclass, even a trivial one.")
+            "subclass, even a trivial one."
+        )
 
     roles, methods = _locate_roles_and_methods(cls)
 
@@ -852,25 +894,30 @@ def _locate_roles_and_methods(cls):
                 continue
 
             # note role declarations
-            if hasattr(method, '_sa_instrument_role'):
+            if hasattr(method, "_sa_instrument_role"):
                 role = method._sa_instrument_role
-                assert role in ('appender', 'remover', 'iterator',
-                                'linker', 'converter')
+                assert role in (
+                    "appender",
+                    "remover",
+                    "iterator",
+                    "linker",
+                    "converter",
+                )
                 roles.setdefault(role, name)
 
             # transfer instrumentation requests from decorated function
             # to the combined queue
             before, after = None, None
-            if hasattr(method, '_sa_instrument_before'):
+            if hasattr(method, "_sa_instrument_before"):
                 op, argument = method._sa_instrument_before
-                assert op in ('fire_append_event', 'fire_remove_event')
+                assert op in ("fire_append_event", "fire_remove_event")
                 before = op, argument
-            if hasattr(method, '_sa_instrument_after'):
+            if hasattr(method, "_sa_instrument_after"):
                 op = method._sa_instrument_after
-                assert op in ('fire_append_event', 'fire_remove_event')
+                assert op in ("fire_append_event", "fire_remove_event")
                 after = op
             if before:
-                methods[name] = before + (after, )
+                methods[name] = before + (after,)
             elif after:
                 methods[name] = None, None, after
     return roles, methods
@@ -892,8 +939,11 @@ def _setup_canned_roles(cls, roles, methods):
         # apply ABC auto-decoration to methods that need it
         for method, decorator in decorators.items():
             fn = getattr(cls, method, None)
-            if (fn and method not in methods and
-                    not hasattr(fn, '_sa_instrumented')):
+            if (
+                fn
+                and method not in methods
+                and not hasattr(fn, "_sa_instrumented")
+            ):
                 setattr(cls, method, decorator(fn))
 
 
@@ -902,26 +952,31 @@ def _assert_required_roles(cls, roles, methods):
     needed
 
     """
-    if 'appender' not in roles or not hasattr(cls, roles['appender']):
+    if "appender" not in roles or not hasattr(cls, roles["appender"]):
         raise sa_exc.ArgumentError(
             "Type %s must elect an appender method to be "
-            "a collection class" % cls.__name__)
-    elif (roles['appender'] not in methods and
-          not hasattr(getattr(cls, roles['appender']), '_sa_instrumented')):
-        methods[roles['appender']] = ('fire_append_event', 1, None)
+            "a collection class" % cls.__name__
+        )
+    elif roles["appender"] not in methods and not hasattr(
+        getattr(cls, roles["appender"]), "_sa_instrumented"
+    ):
+        methods[roles["appender"]] = ("fire_append_event", 1, None)
 
-    if 'remover' not in roles or not hasattr(cls, roles['remover']):
+    if "remover" not in roles or not hasattr(cls, roles["remover"]):
         raise sa_exc.ArgumentError(
             "Type %s must elect a remover method to be "
-            "a collection class" % cls.__name__)
-    elif (roles['remover'] not in methods and
-          not hasattr(getattr(cls, roles['remover']), '_sa_instrumented')):
-        methods[roles['remover']] = ('fire_remove_event', 1, None)
+            "a collection class" % cls.__name__
+        )
+    elif roles["remover"] not in methods and not hasattr(
+        getattr(cls, roles["remover"]), "_sa_instrumented"
+    ):
+        methods[roles["remover"]] = ("fire_remove_event", 1, None)
 
-    if 'iterator' not in roles or not hasattr(cls, roles['iterator']):
+    if "iterator" not in roles or not hasattr(cls, roles["iterator"]):
         raise sa_exc.ArgumentError(
             "Type %s must elect an iterator method to be "
-            "a collection class" % cls.__name__)
+            "a collection class" % cls.__name__
+        )
 
 
 def _set_collection_attributes(cls, roles, methods):
@@ -930,16 +985,20 @@ def _set_collection_attributes(cls, roles, methods):
 
     """
     for method_name, (before, argument, after) in methods.items():
-        setattr(cls, method_name,
-                _instrument_membership_mutator(getattr(cls, method_name),
-                                               before, argument, after))
+        setattr(
+            cls,
+            method_name,
+            _instrument_membership_mutator(
+                getattr(cls, method_name), before, argument, after
+            ),
+        )
     # intern the role map
     for role, method_name in roles.items():
-        setattr(cls, '_sa_%s' % role, getattr(cls, method_name))
+        setattr(cls, "_sa_%s" % role, getattr(cls, method_name))
 
     cls._sa_adapter = None
 
-    if not hasattr(cls, '_sa_converter'):
+    if not hasattr(cls, "_sa_converter"):
         cls._sa_converter = None
     cls._sa_instrumented = id(cls)
 
@@ -949,7 +1008,9 @@ def _instrument_membership_mutator(method, before, argument, after):
     adapter."""
     # This isn't smart enough to handle @adds(1) for 'def fn(self, (a, b))'
     if before:
-        fn_args = list(util.flatten_iterator(inspect_getargspec(method)[0]))
+        fn_args = list(
+            util.flatten_iterator(inspect_getfullargspec(method)[0])
+        )
         if isinstance(argument, int):
             pos_arg = argument
             named_arg = len(fn_args) > argument and fn_args[argument] or None
@@ -966,7 +1027,8 @@ def _instrument_membership_mutator(method, before, argument, after):
             if pos_arg is None:
                 if named_arg not in kw:
                     raise sa_exc.ArgumentError(
-                        "Missing argument %s" % argument)
+                        "Missing argument %s" % argument
+                    )
                 value = kw[named_arg]
             else:
                 if len(args) > pos_arg:
@@ -975,9 +1037,10 @@ def _instrument_membership_mutator(method, before, argument, after):
                     value = kw[named_arg]
                 else:
                     raise sa_exc.ArgumentError(
-                        "Missing argument %s" % argument)
+                        "Missing argument %s" % argument
+                    )
 
-        initiator = kw.pop('_sa_initiator', None)
+        initiator = kw.pop("_sa_initiator", None)
         if initiator is False:
             executor = None
         else:
@@ -1003,7 +1066,11 @@ def _instrument_membership_mutator(method, before, argument, after):
 
 
 def __set(collection, item, _sa_initiator=None):
-    """Run set events, may eventually be inlined into decorators."""
+    """Run set events.
+
+    This event always occurs before the collection is actually mutated.
+
+    """
 
     if _sa_initiator is not False:
         executor = collection._sa_adapter
@@ -1013,15 +1080,22 @@ def __set(collection, item, _sa_initiator=None):
 
 
 def __del(collection, item, _sa_initiator=None):
-    """Run del events, may eventually be inlined into decorators."""
+    """Run del events.
+
+    This event occurs before the collection is actually mutated, *except*
+    in the case of a pop operation, in which case it occurs afterwards.
+    For pop operations, the __before_pop hook is called before the
+    operation occurs.
+
+    """
     if _sa_initiator is not False:
         executor = collection._sa_adapter
         if executor:
             executor.fire_remove_event(item, _sa_initiator)
 
 
-def __before_delete(collection, _sa_initiator=None):
-    """Special method to run 'commit existing value' methods"""
+def __before_pop(collection, _sa_initiator=None):
+    """An event which occurs on a before a pop() operation occurs."""
     executor = collection._sa_adapter
     if executor:
         executor.fire_pre_remove_event(_sa_initiator)
@@ -1038,15 +1112,16 @@ def _list_decorators():
         def append(self, item, _sa_initiator=None):
             item = __set(self, item, _sa_initiator)
             fn(self, item)
+
         _tidy(append)
         return append
 
     def remove(fn):
         def remove(self, value, _sa_initiator=None):
-            __before_delete(self, _sa_initiator)
+            __del(self, value, _sa_initiator)
             # testlib.pragma exempt:__eq__
             fn(self, value)
-            __del(self, value, _sa_initiator)
+
         _tidy(remove)
         return remove
 
@@ -1054,6 +1129,7 @@ def _list_decorators():
         def insert(self, index, value):
             value = __set(self, value)
             fn(self, index, value)
+
         _tidy(insert)
         return insert
 
@@ -1090,10 +1166,12 @@ def _list_decorators():
                     if len(value) != len(rng):
                         raise ValueError(
                             "attempt to assign sequence of size %s to "
-                            "extended slice of size %s" % (len(value),
-                                                           len(rng)))
+                            "extended slice of size %s"
+                            % (len(value), len(rng))
+                        )
                     for i, item in zip(rng, value):
                         self.__setitem__(i, item)
+
         _tidy(__setitem__)
         return __setitem__
 
@@ -1110,16 +1188,19 @@ def _list_decorators():
                 for item in self[index]:
                     __del(self, item)
                 fn(self, index)
+
         _tidy(__delitem__)
         return __delitem__
 
     if util.py2k:
+
         def __setslice__(fn):
             def __setslice__(self, start, end, values):
                 for value in self[start:end]:
                     __del(self, value)
                 values = [__set(self, value) for value in values]
                 fn(self, start, end, values)
+
             _tidy(__setslice__)
             return __setslice__
 
@@ -1128,6 +1209,7 @@ def _list_decorators():
                 for value in self[start:end]:
                     __del(self, value)
                 fn(self, start, end)
+
             _tidy(__delslice__)
             return __delslice__
 
@@ -1135,6 +1217,7 @@ def _list_decorators():
         def extend(self, iterable):
             for value in iterable:
                 self.append(value)
+
         _tidy(extend)
         return extend
 
@@ -1145,24 +1228,28 @@ def _list_decorators():
             for value in iterable:
                 self.append(value)
             return self
+
         _tidy(__iadd__)
         return __iadd__
 
     def pop(fn):
         def pop(self, index=-1):
-            __before_delete(self)
+            __before_pop(self)
             item = fn(self, index)
             __del(self, item)
             return item
+
         _tidy(pop)
         return pop
 
     if not util.py2k:
+
         def clear(fn):
             def clear(self, index=-1):
                 for item in self:
                     __del(self, item)
                 fn(self)
+
             _tidy(clear)
             return clear
 
@@ -1172,7 +1259,7 @@ def _list_decorators():
     # desired.  hard to imagine a use case for __imul__, though.
 
     l = locals().copy()
-    l.pop('_tidy')
+    l.pop("_tidy")
     return l
 
 
@@ -1183,7 +1270,7 @@ def _dict_decorators():
         fn._sa_instrumented = True
         fn.__doc__ = getattr(dict, fn.__name__).__doc__
 
-    Unspecified = util.symbol('Unspecified')
+    Unspecified = util.symbol("Unspecified")
 
     def __setitem__(fn):
         def __setitem__(self, key, value, _sa_initiator=None):
@@ -1191,6 +1278,7 @@ def _dict_decorators():
                 __del(self, self[key], _sa_initiator)
             value = __set(self, value, _sa_initiator)
             fn(self, key, value)
+
         _tidy(__setitem__)
         return __setitem__
 
@@ -1199,6 +1287,7 @@ def _dict_decorators():
             if key in self:
                 __del(self, self[key], _sa_initiator)
             fn(self, key)
+
         _tidy(__delitem__)
         return __delitem__
 
@@ -1207,26 +1296,32 @@ def _dict_decorators():
             for key in self:
                 __del(self, self[key])
             fn(self)
+
         _tidy(clear)
         return clear
 
     def pop(fn):
         def pop(self, key, default=Unspecified):
-            if key in self:
-                __del(self, self[key])
+            __before_pop(self)
+            _to_del = key in self
             if default is Unspecified:
-                return fn(self, key)
+                item = fn(self, key)
             else:
-                return fn(self, key, default)
+                item = fn(self, key, default)
+            if _to_del:
+                __del(self, item)
+            return item
+
         _tidy(pop)
         return pop
 
     def popitem(fn):
         def popitem(self):
-            __before_delete(self)
+            __before_pop(self)
             item = fn(self)
             __del(self, item[1])
             return item
+
         _tidy(popitem)
         return popitem
 
@@ -1237,16 +1332,16 @@ def _dict_decorators():
                 return default
             else:
                 return self.__getitem__(key)
+
         _tidy(setdefault)
         return setdefault
 
     def update(fn):
         def update(self, __other=Unspecified, **kw):
             if __other is not Unspecified:
-                if hasattr(__other, 'keys'):
+                if hasattr(__other, "keys"):
                     for key in list(__other):
-                        if (key not in self or
-                                self[key] is not __other[key]):
+                        if key not in self or self[key] is not __other[key]:
                             self[key] = __other[key]
                 else:
                     for key, value in __other:
@@ -1255,14 +1350,16 @@ def _dict_decorators():
             for key in kw:
                 if key not in self or self[key] is not kw[key]:
                     self[key] = kw[key]
+
         _tidy(update)
         return update
 
     l = locals().copy()
-    l.pop('_tidy')
-    l.pop('Unspecified')
+    l.pop("_tidy")
+    l.pop("Unspecified")
     return l
 
+
 _set_binop_bases = (set, frozenset)
 
 
@@ -1274,8 +1371,10 @@ def _set_binops_check_strict(self, obj):
 
 def _set_binops_check_loose(self, obj):
     """Allow anything set-like to participate in set binops."""
-    return (isinstance(obj, _set_binop_bases + (self.__class__,)) or
-            util.duck_type_collection(obj) == set)
+    return (
+        isinstance(obj, _set_binop_bases + (self.__class__,))
+        or util.duck_type_collection(obj) == set
+    )
 
 
 def _set_decorators():
@@ -1285,7 +1384,7 @@ def _set_decorators():
         fn._sa_instrumented = True
         fn.__doc__ = getattr(set, fn.__name__).__doc__
 
-    Unspecified = util.symbol('Unspecified')
+    Unspecified = util.symbol("Unspecified")
 
     def add(fn):
         def add(self, value, _sa_initiator=None):
@@ -1293,6 +1392,7 @@ def _set_decorators():
                 value = __set(self, value, _sa_initiator)
             # testlib.pragma exempt:__hash__
             fn(self, value)
+
         _tidy(add)
         return add
 
@@ -1303,6 +1403,7 @@ def _set_decorators():
                 __del(self, value, _sa_initiator)
                 # testlib.pragma exempt:__hash__
             fn(self, value)
+
         _tidy(discard)
         return discard
 
@@ -1313,15 +1414,19 @@ def _set_decorators():
                 __del(self, value, _sa_initiator)
             # testlib.pragma exempt:__hash__
             fn(self, value)
+
         _tidy(remove)
         return remove
 
     def pop(fn):
         def pop(self):
-            __before_delete(self)
+            __before_pop(self)
             item = fn(self)
+            # for set in particular, we have no way to access the item
+            # that will be popped before pop is called.
             __del(self, item)
             return item
+
         _tidy(pop)
         return pop
 
@@ -1329,6 +1434,7 @@ def _set_decorators():
         def clear(self):
             for item in list(self):
                 self.remove(item)
+
         _tidy(clear)
         return clear
 
@@ -1336,6 +1442,7 @@ def _set_decorators():
         def update(self, value):
             for item in value:
                 self.add(item)
+
         _tidy(update)
         return update
 
@@ -1346,6 +1453,7 @@ def _set_decorators():
             for item in value:
                 self.add(item)
             return self
+
         _tidy(__ior__)
         return __ior__
 
@@ -1353,6 +1461,7 @@ def _set_decorators():
         def difference_update(self, value):
             for item in value:
                 self.discard(item)
+
         _tidy(difference_update)
         return difference_update
 
@@ -1363,6 +1472,7 @@ def _set_decorators():
             for item in value:
                 self.discard(item)
             return self
+
         _tidy(__isub__)
         return __isub__
 
@@ -1375,6 +1485,7 @@ def _set_decorators():
                 self.remove(item)
             for item in add:
                 self.add(item)
+
         _tidy(intersection_update)
         return intersection_update
 
@@ -1390,6 +1501,7 @@ def _set_decorators():
             for item in add:
                 self.add(item)
             return self
+
         _tidy(__iand__)
         return __iand__
 
@@ -1402,6 +1514,7 @@ def _set_decorators():
                 self.remove(item)
             for item in add:
                 self.add(item)
+
         _tidy(symmetric_difference_update)
         return symmetric_difference_update
 
@@ -1417,12 +1530,13 @@ def _set_decorators():
             for item in add:
                 self.add(item)
             return self
+
         _tidy(__ixor__)
         return __ixor__
 
     l = locals().copy()
-    l.pop('_tidy')
-    l.pop('Unspecified')
+    l.pop("_tidy")
+    l.pop("Unspecified")
     return l
 
 
@@ -1446,18 +1560,17 @@ __canned_instrumentation = {
 
 __interfaces = {
     list: (
-        {'appender': 'append', 'remover': 'remove',
-         'iterator': '__iter__'}, _list_decorators()
+        {"appender": "append", "remover": "remove", "iterator": "__iter__"},
+        _list_decorators(),
+    ),
+    set: (
+        {"appender": "add", "remover": "remove", "iterator": "__iter__"},
+        _set_decorators(),
     ),
-
-    set: ({'appender': 'add',
-           'remover': 'remove',
-           'iterator': '__iter__'}, _set_decorators()
-          ),
-
     # decorators are required for dicts and object collections.
-    dict: ({'iterator': 'values'}, _dict_decorators()) if util.py3k
-    else ({'iterator': 'itervalues'}, _dict_decorators()),
+    dict: ({"iterator": "values"}, _dict_decorators())
+    if util.py3k
+    else ({"iterator": "itervalues"}, _dict_decorators()),
 }
 
 
@@ -1508,33 +1621,10 @@ class MappedCollection(dict):
                 "Can not remove '%s': collection holds '%s' for key '%s'. "
                 "Possible cause: is the MappedCollection key function "
                 "based on mutable properties or properties that only obtain "
-                "values after flush?" %
-                (value, self[key], key))
+                "values after flush?" % (value, self[key], key)
+            )
         self.__delitem__(key, _sa_initiator)
 
-    @collection.converter
-    def _convert(self, dictlike):
-        """Validate and convert a dict-like object into values for set()ing.
-
-        This is called behind the scenes when a MappedCollection is replaced
-        entirely by another collection, as in::
-
-          myobj.mappedcollection = {'a':obj1, 'b': obj2} # ...
-
-        Raises a TypeError if the key in any (key, value) pair in the dictlike
-        object does not match the key that this collection's keyfunc would
-        have assigned for that value.
-
-        """
-        for incoming_key, value in util.dictlike_iteritems(dictlike):
-            new_key = self.keyfunc(value)
-            if incoming_key != new_key:
-                raise TypeError(
-                    "Found incompatible key %r for value %r; this "
-                    "collection's "
-                    "keying function requires a key of %r for this value." % (
-                        incoming_key, value, new_key))
-            yield value
 
 # ensure instrumentation is associated with
 # these built-in classes; if a user-defined class
